import { act, render, screen, waitFor } from '@testing-library/react-native'
import { ClientEvent } from 'matrix-js-sdk'
import { ICreateClientOpts } from 'matrix-js-sdk/src/client'
import React from 'react'

import ChatHomeContent from '@holi/chat/src/components/ChatHomeContent'
import ChatProvider from '@holi/chat/src/components/ChatProvider'
import ChatSessionInitializer from '@holi/chat/src/components/ChatSessionInitializer'
import { ChatMockProvider, queriesFor, storeForRoomAtoms } from '@holi/chat/src/test-utils/ChatMockProvider'
import {
  createChatMemberFixture,
  createChatMessageFixture,
  createChatRoomFixture,
} from '@holi/chat/src/test-utils/matrix-fixtures'
import {
  CHAT_MOCK_USER_ID,
  mockCreateMatrixClient,
  mockMx,
  setMatrixMock,
} from '@holi/chat/src/test-utils/matrix-mock-setup'
import { fireMatrixEvent, matrixRoomFrom } from '@holi/chat/src/test-utils/matrix-utils'
import { markDeviceTokenAsInitialized } from '@holi/core/screens/notifications/helpers/useDeviceToken'
import { User } from '@holi/core/screens/userprofile/types'

interface MockLoggedInUserState {
  isLoggedIn: boolean
  isLoading: boolean
}

let mockLoggedInUser: User | undefined = undefined
// eslint-disable-next-line @typescript-eslint/no-unused-vars
let mockOrySessionInStorage: Record<string, string> | undefined
let mockMatrixSessionInStorage: [string, boolean | string | undefined][]
let mockChatFeatureToggle: Record<string, boolean | string>
let mockLoggedInUserState: MockLoggedInUserState | undefined = {
  isLoggedIn: false,
  isLoading: false,
}

jest.mock('@holi/core/screens/userprofile/useLoggedInUser', () => ({
  useLoggedInUser: () => ({
    user: mockLoggedInUser,
    ...mockLoggedInUserState,
    refetch: jest.fn(),
  }),
}))

jest.mock('@holi/core/helpers/useFeatureToggle', () => ({
  useFeatureToggle: () => mockChatFeatureToggle,
  FeatureToggleName: {
    CHAT: 'CHAT',
  },
}))

jest.mock('@react-native-async-storage/async-storage', () => {
  return {
    multiGet: async () => mockMatrixSessionInStorage,
    multiSet: async () => mockMatrixSessionInStorage,
    multiRemove: async () => null,
  }
})

jest.mock('matrix-js-sdk', () => ({
  ...jest.requireActual('matrix-js-sdk'),
  createClient: (opts: ICreateClientOpts) => mockCreateMatrixClient(opts.userId ?? '', mockMx),
}))

describe('ChatHome', () => {
  beforeEach(() => {
    markDeviceTokenAsInitialized()
    mockChatFeatureToggle = {
      ftLoading: false,
      ftError: false,
      ftStatus: 'FeatureToggleStatus.PRODUCTION',
      ftEnabled: true,
    }
  })

  afterEach(() => {
    mockOrySessionInStorage = undefined
    mockLoggedInUser = undefined
    mockLoggedInUserState = undefined
    mockMatrixSessionInStorage = []
  })

  describe('for guests', () => {
    beforeAll(() => {
      mockOrySessionInStorage = {}
      mockLoggedInUser = undefined
      mockLoggedInUserState = {
        isLoggedIn: false,
        isLoading: false,
      }
      mockMatrixSessionInStorage = [
        ['mx_client_login', undefined],
        ['mx_user_id', undefined],
        ['mx_sso_token', undefined],
        ['mx_device', undefined],
      ]
    })

    it('shows login hint', async () => {
      render(
        <ChatSessionInitializer>
          <ChatProvider>
            <ChatHomeContent />
          </ChatProvider>
        </ChatSessionInitializer>
      )

      expect(await screen.findByText('loginRedirect.title')).toBeOnTheScreen()
      expect(await screen.findByText('loginRedirect.text')).toBeOnTheScreen()
    })
  })

  describe('for signed-in user', () => {
    beforeEach(() => {
      mockOrySessionInStorage = {
        session_token: 'session_token_stub',
      }
      mockLoggedInUser = {
        id: 'someLoggedInUserId',
        avatarDefaultColor: '',
        avatarLabel: '',
        fullName: '',
        identity: 'someLoggedInUserIdentity',
      }
      mockLoggedInUserState = {
        isLoggedIn: true,
        isLoading: false,
      }
      mockMatrixSessionInStorage = [
        ['mx_client_login', true], //fresh?
        ['mx_user_id', CHAT_MOCK_USER_ID],
        ['mx_sso_token', 'someAccessToken'],
        ['mx_device', 'someDeviceId'],
      ]
      mockChatFeatureToggle = {
        ftLoading: false,
        ftError: false,
        ftStatus: 'FeatureToggleStatus.PRODUCTION',
        ftEnabled: true,
      }

      setMatrixMock(CHAT_MOCK_USER_ID)
    })

    it('shows empty home chat view', async () => {
      render(
        <ChatSessionInitializer>
          <ChatProvider>
            <ChatHomeContent />
          </ChatProvider>
        </ChatSessionInitializer>
      )

      await waitFor(() => {
        expect(screen.getByText('chat.empty.tagline')).toBeOnTheScreen()
        expect(screen.getByText('chat.empty.headline')).toBeOnTheScreen()
        expect(screen.getByText('chat.empty.button.text')).toBeOnTheScreen()
      })
    })

    it('shows an existing room', async () => {
      // GIVEN a pre-existing room with a message by bob
      const bob = createChatMemberFixture({ name: 'Bob' })
      const bobsRoom = createChatRoomFixture({
        // we use user identities as room names?
        name: 'BobIdentity',
        creator: bob,
      })
      bobsRoom.lastMessage = createChatMessageFixture('Moin!', bob)

      const members = [bob]
      const rooms = [bobsRoom]

      // WHEN rendering the chat home
      render(
        <ChatMockProvider jotaiStore={storeForRoomAtoms(rooms)} apolloMocks={queriesFor(rooms, members)}>
          <ChatHomeContent />
        </ChatMockProvider>
      )

      // THEN it shows the room list
      expect(await screen.findByTestId('chat-room-list')).toBeOnTheScreen()

      // THEN intermediately a skeleton for the room is shown
      // TODO FIXME: this should test if a skeleton is shown. the way the test setup is though, the room get's rendered immediately as the store already contains the room
      //expect(await screen.findByTestId('room-item-BobIdentity-skeleton')).toBeOnTheScreen()
      //expect(await screen.findByTestId('room-item-BobIdentity-skeleton')).not.toBeOnTheScreen()

      // THEN bobs room is shown in a list and the last message is displayed
      const listedRoom = await screen.findByTestId('room-item-BobIdentity')
      expect(listedRoom).toHaveTextContent('chat.invitation.newconversation')
      expect(listedRoom).toHaveTextContent('Moin!')
    })

    it('shows an incoming room', async () => {
      const aliceRoom = createChatRoomFixture({
        // we use user identities as room names?
        name: 'AliceIdentity',
      })

      render(
        <ChatMockProvider>
          <ChatHomeContent />
        </ChatMockProvider>
      )

      // wait for chat to be initialized before firing an event
      expect(await screen.findByText('chat.empty.tagline')).toBeOnTheScreen()

      act(() => {
        fireMatrixEvent(mockMx.on, ClientEvent.Room, matrixRoomFrom(aliceRoom))
      })

      expect(await screen.findByTestId('chat-room-list')).toBeOnTheScreen()
      expect(await screen.findByTestId('room-item-AliceIdentity')).toBeOnTheScreen()
    })
  })
})
