import * as React from 'react'
import { AccessibilityProps, Platform, StyleSheet, Text, TextProps, View } from 'react-native'

import type { AccessibilityPropsWeb } from '@holi/ui/@types/reactNativeWeb'
import HoliSkeleton from '@holi/ui/components/atoms/HoliSkeleton'
import { defaultTextOptionsAsProps } from '@holi/ui/components/atoms/HoliSkeleton/defaults'
import { SkeletonTextOptions } from '@holi/ui/components/atoms/HoliSkeleton/types'
import { colors } from '@holi/ui/styles/globalVars'
import { HoliTheme, useTheme } from '@holi/ui/styles/theme'

type HoliHeadingLevel = '1' | '2' | '3' | '4' // for working code completion in vscode
type HoliHeadingSize = 'super' | 'large' | 'medium' | 'small' | 'xSmall' // for working code completion in vscode

const HoliHeadings: Record<
  HoliHeadingLevel,
  {
    defaultSize: HoliHeadingSize
  }
> = {
  1: {
    defaultSize: 'large',
  },
  2: {
    defaultSize: 'medium',
  },
  3: {
    defaultSize: 'small',
  },
  4: {
    defaultSize: 'xSmall',
  },
}

export const HoliHeadingLevels: HoliHeadingLevel[] = Object.keys(HoliHeadings) as unknown as HoliHeadingLevel[]
export const HoliHeadingSizes: HoliHeadingSize[] = ['super', 'large', 'medium', 'small'] // keep in sync with type HoliHeadingSize

export type HoliHeadingProps = {
  size?: HoliHeadingSize
  level?: HoliHeadingLevel
  selectable?: boolean
  testID?: string
  numberOfLines?: number
  loading?: boolean
  skeletonOptions?: SkeletonTextOptions
}

const HoliHeading = ({
  children,
  level = '2',
  size = HoliHeadings[level].defaultSize,
  selectable = true,
  testID,
  numberOfLines,
  loading = false,
  skeletonOptions,
  ...props
}: React.PropsWithChildren<HoliHeadingProps & TextProps>) => {
  const { theme } = useTheme()
  const styles = getStyles(theme)

  if (loading) {
    return (
      <View style={props?.style}>
        <HoliSkeleton {...defaultTextOptionsAsProps(skeletonOptions, numberOfLines, styles[size].lineHeight, '40%')} />
      </View>
    )
  }

  const accessibilityProps: AccessibilityProps & AccessibilityPropsWeb = Platform.select({
    web: {
      role: 'heading',
      'aria-level': Number(level),
    },
    default: {
      role: 'heading',
    },
  })

  return (
    <Text
      {...props}
      {...accessibilityProps}
      selectable={Platform.OS !== 'web' ? selectable : undefined}
      style={StyleSheet.flatten([
        { userSelect: selectable ? 'auto' : 'none' },
        styles.default,
        styles[size as keyof typeof styles],
        props?.style,
      ])}
      testID={testID}
      numberOfLines={numberOfLines}
    >
      {children}
    </Text>
  )
}

export default HoliHeading

const getStyles = (theme: HoliTheme) =>
  StyleSheet.create({
    default: {
      fontWeight: '500',
      fontFamily: theme.fontFamily.default('Medium'),
      color: colors.foreground30,
    },
    super: {
      fontSize: 36,
      lineHeight: 36 * (Platform.OS === 'ios' ? 1.4 : 1.2),
    },
    large: {
      fontSize: 28,
      lineHeight: 28 * (Platform.OS === 'ios' ? 1.4 : 1.2),
    },
    medium: {
      fontSize: 21,
      lineHeight: 21 * (Platform.OS === 'ios' ? 1.4 : 1.2),
    },
    small: {
      fontSize: 16,
      lineHeight: 16 * 1.4,
    },
    xSmall: {
      fontSize: 14,
      lineHeight: 14 * 1.4,
    },
  })
